# coding: utf-8
from __future__ import unicode_literals, absolute_import
from boxsdk.object.collaboration_whitelist_exempt_target import CollaborationWhitelistExemptTarget


class CollaborationAllowlistExemptTarget(CollaborationWhitelistExemptTarget):
    """Represents a user who is exempted from the collaboration allowlist."""

    _item_type = 'collaboration_whitelist_exempt_target'
