# coding: utf-8
from __future__ import unicode_literals, absolute_import

from .base_object import BaseObject


class FileVersionRetention(BaseObject):
    """Represents a Box file version retention."""
    _item_type = 'file_version_retention'
